(* Record the RTT to the sequencer. *)

open Ensemble
open View
open Event
open Util
open Layer
open Trans

open Unix
(**************************************************************)
let name = Trace.filel "Chk_total_latency"
(**************************************************************)

type header = NoHdr | TS of float

type state = {
    seq_rank	      : rank ;		(* sequencer rank *)
    mutable logs      : float list; 
  } 
      
(**************************************************************)
open Sticky

let init _ ((ls,vs) as vf) = 
  { seq_rank      = Sticky.sequencer_rank vf;
    logs = [] }
    
let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dnnm} =
  let failwith m = failwith (name^":"^m) in
  let am_sequencer () = (ls.rank = s.seq_rank) in
  let dump_logs () = 
    let dump_log f log = 
      let moments l = 
	let bi_sum (m1,m2) n = (m1 +. n, m2 +. (n *. n)) in 
	List.fold_left bi_sum (0.0, 0.0) l
      in
      let n = (List.length log) in 
      let (m1,m2) = moments log in 
      let mean = m1 /. (float n) in
      let var = m2 /. (float n) -. mean *. mean in
      Printf.fprintf f "# [n= %d, mean= %f, variance= %f]\n" n mean var;
      List.iter (fun t -> Printf.fprintf f "%f\n" t) log;
      Printf.fprintf f "\n\n"; (* New data set -- gnuplot convention *)
    in

    if List.length s.logs > 0 then 
      begin
	let log_name = Param.string vs.params "chk_total_latency_log" in
	let f = open_out_gen [Open_wronly;Open_creat;Open_append] 0o666 log_name in
	Util.eprintf "\nDumping RTT sequencer stats in %s\n" log_name;
	Printf.fprintf f "#### %s ####\n" (View.string_of_id ls.view_id);
	dump_log f (List.rev s.logs);
	close_out f
      end
  in

  let up_hdlr ev abv hdr =
    match getType ev, hdr with 
    | ECast, (TS ts) ->
        if getPeer ev = ls.rank then
          s.logs <- (Unix.gettimeofday () -. ts) :: s.logs;
	up ev abv
    | _ -> up ev abv
  and uplm_hdlr ev hdr =  failwith "Unknown local message"
  and upnm_hdlr ev =
    match getType ev with
    | EView ->
	dump_logs();
	upnm ev
    | _ -> upnm ev

  and dn_hdlr ev abv = 
    match getType ev, getApplMsg ev, getNoTotal ev, (am_sequencer()) with
    | ECast, true, false, false -> 
	dn ev abv (TS (Unix.gettimeofday()))
    | _ -> dn ev abv NoHdr
  and dnnm_hdlr = dnnm

  in {up_in=up_hdlr;uplm_in=uplm_hdlr;upnm_in=upnm_hdlr;dn_in=dn_hdlr;dnnm_in=dnnm_hdlr}

let l args vs = Layer.hdr init hdlrs None NoOpt args vs

let _ =  
  Param.default "chk_total_latency_log" (Param.String "sequencer_latency");
  Elink.layer_install name l





