(* Provide the rank of the sequencer, taking account user specifications *)

(* The sequencer of the group will be, if possible on the node
  specified by the `sequencer_sticky_name' stack parameter.

  `sequencer_sticky_name' is compared either against the Endpt.id or
   against the IP Address member (require Ensemble UDP mode to be
   set). *)
module Sticky = struct
  open Ensemble
  open Layer
  open View
  open Event
  open Util
  open Trans

  exception FoundSequencer of rank

   (* Who is the sequencer *)
  let sequencer_rank (ls,vs) =
    let name = Trace.file "STICKY" in
    let log = Trace.log2 name ls.name in
    let application_choice =
      try 
	let name =  (Param.string vs.params "sequencer_sticky_name") in 
	Some (Hsys.inet_of_string  name)
      with _ ->
	None
    in
    match application_choice with
    | Some (first_choice) ->
	
	 (* Take the lowest rank matching first_choice *)
	(try
	  Arrayf.iteri 
	    (fun rank addr_set ->

	      let adr = Addr.project addr_set Addr.Udp in
	      match adr with 
	      |	Addr.UdpA (inet,port)->
		  log (fun () -> sprintf "Looking %s" (Hsys.string_of_inet inet));
		  if inet = first_choice then 
		    raise (FoundSequencer rank)
	      |	_ -> 
		  log (fun () -> sprintf "Need UDP transport modes to stick the sequencer.") )
	    vs.address;

	  log (fun () -> 
	    sprintf "No matching member for the sequencer (%s). Default to zero" 
	      (Hsys.string_of_inet first_choice));

	   (* The first choice is not in the view. Default to the coordinator *)
	  vs.coord
	with (FoundSequencer seq) -> 
	  log (fun () -> sprintf "Sequencer is member %d" seq);
	  seq ) 
    | None ->
	 (* No first choice. Default to coordinator *)
	vs.coord

end
