(**************************************************************)
(* Voyeur.ml : Trace all Ensemble events passed up and down          *) 
(**************************************************************)

open View
open Event
open Util
open Layer
open Trans
(**************************************************************)
let name = Trace.filel "Voyeur"
(**************************************************************)

type state = { mutable nb : int }
      
let init _ (ls,vs) = 
  let log = Trace.log2 name ls.name in
  log (fun () -> sprintf "init: view= %s" (View.string_of_id ls.view_id));
  {nb = 0}
    
let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dnnm} =
  let logEv way ev =  
    let log = Trace.log2 name ls.name in
    log (fun () ->  sprintf "%s(%s)" way (Event.string_of_type (getType ev)));
    s.nb <- succ s.nb
  in

  let up_hdlr ev abv hdr = logEv "Up" ev; up ev abv
  and uplm_hdlr ev hdr   = logEv "Up" ev; free name ev
  and upnm_hdlr ev       = logEv "Up" ev; upnm ev
  and dn_hdlr ev abv     = logEv "Dn" ev; dn ev abv ()
  and dnnm_hdlr ev       = logEv "Dn" ev; dnnm ev

  in {up_in=up_hdlr;uplm_in=uplm_hdlr;upnm_in=upnm_hdlr;dn_in=dn_hdlr;dnnm_in=dnnm_hdlr}

let l args vs = Layer.hdr init hdlrs None NoOpt args vs

let _ = Elink.layer_install name l
