(* Basic interfaces types for LESS protocols *)

(* The interface of a replica *) 
module type ADT = sig
  type op_t
  type state

  val install_state : state -> unit
  val get_state : unit -> state

  val execute : op_t -> bool
  val to_string : op_t -> string 
end      

(* The metrics used by the divergence protocol *)
module type Metrics_intf = sig 
  type t

  val null : unit -> t 
  val add : t -> t -> t
  val sub : t -> t -> t
  val gt  : t -> t -> bool
  val to_string : t -> string
end

(* The measure interface used by the divergence protocol *)
module type Measure_intf = sig
  type op_t
  type metrics
  val mu : op_t -> metrics  
end


module type Measure = 
  functor (AD : ADT) -> 
    functor (MI : Metrics_intf) -> 
      Measure_intf with type op_t = AD.op_t and type metrics = MI.t


(* A straigth forward metrics implementation *)
module Scalar_metrics = struct 
  open Printf 

  type t = int 
	
  let null () = 0
  let add m1 m2  = m1 + m2
  let sub m1 m2  = m1 - m2
  let gt m1 m2  = m1 > m2
  let to_string m = sprintf "%d" m
end

