module type Downcalls_intf = sig
  open Ensemble
  open Appl_intf 
  open New

  type op_t
  type operation
  type acker   = op_t -> unit
  type issuer = (op_t * acker) -> (operation naction) list 
end

    
module type Protocol_intf = sig
  open Ensemble
  open Appl_intf 
  open New
  open Trans
  open Util

  exception Pending_request		(* current implementation requires synchronous operations *)

  type op_t
  type metrics
  type operation
  type acker  = op_t -> unit
  type issuer = (op_t * acker) -> (operation naction) list 

  type handlers = { 
      deliver : origin -> cast_or_send -> operation -> (operation naction) list;
      issue   : issuer;
      dump    : unit -> unit;
      stable  : unit -> bool;
    }

  val install : 
      View.full -> bool -> int -> metrics ->
	(operation naction list) * handlers
end


open Measure_intf
	      
module type Protocol = 
    functor (A : ADT) ->
      functor (MI : Metrics_intf) ->
	functor (M : Measure_intf with type op_t = A.op_t and type metrics = MI.t) -> 
	  Protocol_intf with type op_t = A.op_t and type metrics = MI.t
