#!/bin/sh
# Generates run scripts for experiments

nb () {
  echo $#
}

nb_op=5000			# each writer emits nb_op

readers="zatar"
writers="tif"
members="$readers $writers"

nb_members=`nb $members`
nb_writers=`nb $writers`

oprates_seq="100 120 140"
sequencer="onion.lcs.mit.edu"	# long name! and should be one of the writer

oprates_div="5 10 20 40 60 80 100 120 140"
threshold="1 2 3 4 5 10 20"

program="./bench-stall.skt -noqueue"
commonargs="-multiread -init 100000 -consumer -sequencer $sequencer"

# Total number of operation to execute
let nb_exec=nb_op*nb_writers


## Here we go !
cat <<@@@
#!/bin/sh
# Run script automatically generated by makerun.sh
set -x
mkdir \$SHORTHOST

is_reader (){
  expr $readers : ".*\$1.*" >> /dev/null
  return 
}

is_writer (){
  expr $writers : ".*\$1.*" >> /dev/null
  return 
}

# Divergence Benches
for rate in $oprates_div; do
  for t in $threshold; do 
    if expr \$rate \* \$t '>=' 100 > /dev/null; then
      id="l-\${t}-\${rate}-${nb_writers}-${nb_members}-nq/"
      logname="\$SHORTHOST/\$id"
      rm -rf \$logname
      mkdir \$logname
      if is_reader \$SHORTHOST; then
	$program $commonargs -oprate \$rate -epsilon 100000 -threshold \$t -n $nb_members -bench_id \$logname -id \$id -nbop 0 -exec $nb_exec  2>&1 | tee \${logname}trace
      else
	$program $commonargs -oprate \$rate -epsilon 100000 -threshold \$t -n $nb_members -bench_id \$logname -id \$id -nbop $nb_op -exec $nb_exec  2>&1 | tee \${logname}trace
      fi
    fi
  done
done

# Sequencer Benches
for rate in $oprates_seq; do
  id="l-0-\${rate}-${nb_writers}-${nb_members}-nq/"
  logname="\$SHORTHOST/\$id"
  rm -rf \$logname
  mkdir \$logname
  if is_reader \$SHORTHOST; then  
    $program $commonargs -oprate \$rate -epsilon 0 -n $nb_members -bench_id \$logname -id \$id -nbop 0 -exec $nb_exec  2>&1 | tee \${logname}trace
  else
    $program $commonargs -oprate \$rate -epsilon 0 -n $nb_members -bench_id \$logname -id \$id -nbop $nb_op -exec $nb_exec  2>&1 | tee \${logname}trace
  fi
done
@@@

