#!/bin/sh

set -x

program=./bench.skt
commonargs="-multiread -init 100000 -consumer"


# Maximal operations rate with three members (atropos,lachesis,tif)
# and two writers (lachesis,tif) case. The two writers are remote. The
# sequencer is on tif. The meaninful logs are on lachesis.

if [ `hostname` != "onion" ]; then 
  # Sequencer Benches
  oprates="110 120 130 140"
  for rate in $oprates; do
    logname="l-0-${rate}-2"
    rm -rf $logname
    mkdir $logname
    if [ `hostname` = "atropos" ]; then  
      $program $commonargs "-sequencer tif.inria.fr" -oprate $rate -epsilon 0 -n 3 -bench_id $logname/ -id $logname -nbop 0 -exec 10000 
    else
      $program $commonargs "-sequencer tif.inria.fr" -oprate $rate -epsilon 0 -n 3 -bench_id $logname/ -id $logname  -nbop 5000 -exec 10000
    fi
  done

  # Divergence Benches
  oprates="20 40 60 80 100 120 160"
  threshold="1 2 3 4 5 6 7"
  for rate in $oprates; do
    for t in $threshold; do 
      logname="l-${t}-${rate}-2"
      rm -rf $logname
      mkdir $logname
      if [ `hostname` = "atropos" ]; then
	$program $commonargs -oprate $rate -epsilon 1000 -threshold $t -n 3 -bench_id $logname/ -id $logname -nbop 0 -exec 10000 
      else
	$program $commonargs -oprate $rate -epsilon 1000 -threshold $t -n 3 -bench_id $logname/ -id $logname  -nbop 5000 -exec 10000
      fi
    done
  done
fi



# Comparison between sequencer and divergence control with three
# members (onion,lachesis,tif) and two writers (lachesis,tif). The two
# writers are remote. The sequencer is on lachesis. The meaninful logs
# are on tif and onion.

if [ `hostname` != "atropos" ]; then 
  # Sequencer Benches
  oprates="120 140"
  commonargs="-multiread -init 100000 -consumer -sequencer lachesis.cs.brandeis.edu"
  for rate in $oprates; do
    logname="lc-0-${rate}-2"
    rm -rf $logname
    mkdir $logname
    if [ `hostname` = "lachesis" ]; then  
      $program $commonargs -oprate $rate -epsilon 0 -n 3 -bench_id $logname/ -id $logname -nbop 0 -exec 10000 
    else
      $program $commonargs -oprate $rate -epsilon 0 -n 3 -bench_id $logname/ -id $logname  -nbop 5000 -exec 10000
    fi
  done

  # Divergence Benches
  oprates="20 40 60 80 100 120 160"
  threshold="1 2 3 4 5 6"
  commonargs="-multiread -init 100000 -consumer"
  for rate in $oprates; do
    for t in $threshold; do 
      logname="l-${t}-${rate}-2"
      rm -rf $logname
      mkdir $logname
      if [ `hostname` = "lachesis" ]; then
	$program $commonargs -oprate $rate -epsilon 1000 -threshold $t -n 3 -bench_id $logname/ -id $logname -nbop 0 -exec 10000 
      else
	$program $commonargs -oprate $rate -epsilon 1000 -threshold $t -n 3 -bench_id $logname/ -id $logname  -nbop 5000 -exec 10000
      fi
    done
  done
fi
