open Chrono
open Printf

let timeStamp = 
  let timeStamp = ref 1 in 
  (fun () -> 
    let ts = !timeStamp in 
    incr timeStamp;
    ts)
      
module TimeStampTimeSet = struct
  type time = int

  let null  _ = 0
  let getTime = timeStamp
  let sub t1 t2 = t1-t2
  let add t1 t2 = t1-t2

  let to_string t = sprintf "%d" t
end

module BasicChrono = Chrono(TimeStampTimeSet)

let test3 () = 
  let myChronos = BasicChrono.create 3 in 
  let startChronos () =
    for i = 0 to ((Array.length myChronos) - 1) do
      BasicChrono.start myChronos.(i)
    done
  in
  startChronos ();
  Array.iteri 
    (fun i chrono -> 
      Printf.eprintf "chrono %d : %s\n" 
	i (TimeStampTimeSet.to_string (BasicChrono.getTime chrono)) )
    myChronos;

  timeStamp(); timeStamp();
  BasicChrono.stop myChronos.(2);
  timeStamp(); timeStamp();
  BasicChrono.stop myChronos.(1);
  timeStamp(); timeStamp();
  BasicChrono.stop myChronos.(0);

  Array.iteri 
    (fun i chrono -> 
      Printf.eprintf "chrono %d : %s\n" 
	i (TimeStampTimeSet.to_string (BasicChrono.getTime chrono)) )
    myChronos


let test2 () = 
  let argv = Array.to_list Sys.argv in
  let (pname::argvl) = argv in 
  let arg_list = (List.map (fun arg -> int_of_string arg) argvl) in 
  let argc = List.length arg_list in
  let myChronos = BasicChrono.create argc in 
  let startChronos () =
    for i = 0 to ((Array.length myChronos) - 1) do
      BasicChrono.start myChronos.(i)
    done
  in
  let rec foo l = 
    match l with 
    | [] ->
	(fun () ->eprintf "."; timeStamp();())
    | hd :: tl -> 
	let len = List.length l in
	let f = foo tl in 
	((fun x () ->
	  eprintf "%d %d\n" len x;
	  for i = 1 to x do
	    f ();
	  done;
	  BasicChrono.stop myChronos.(len - 1);
	  eprintf "\n") hd)
  in
  let bench = foo arg_list in 
  startChronos ();
  bench ();
  Array.iteri 
    (fun i chrono -> 
      Printf.eprintf "chrono %d : %s\n" 
	i (TimeStampTimeSet.to_string (BasicChrono.getTime chrono)) )
    myChronos


open UnixChrono

let test1 () = 
  let argv = Array.to_list Sys.argv in
  let (pname::argvl) = argv in 
  let arg_list = (List.map (fun arg -> int_of_string arg) argvl) in 
  let argc = List.length arg_list in
  let myChronos = UnixChrono.create argc in 
  let startChronos () =
    for i = 0 to ((Array.length myChronos) - 1) do
      UnixChrono.start myChronos.(i)
    done
  in
  let rec foo l = 
    match l with 
    | [] ->
	(fun () -> Unix.getpid();())
    | hd :: tl -> 
	let len = List.length l in
	let f = foo tl in 
	((fun x () ->
	  for i = 1 to x do f () done;
	  UnixChrono.stop myChronos.(len - 1)) hd)
  in
  let bench = foo arg_list in 
  startChronos ();
  bench ();
  Array.iteri 
    (fun i chrono -> 
      Printf.eprintf "chrono %d : %s\n" 
	i (UnixTimeSet.to_string (UnixChrono.getTime chrono)) )
    myChronos


let _ = test1 ()

