(* Causal time stamp. Ensemble MCausal are integrate acknowledgement. *)

module Causalts = struct 
  type seqno = int
  type s = seqno array			(* Causally received seqno per member *)

  let create_state nmem = Array.create nmem 0
      
(* less-than *)
  let lt t1 t2 = 
    let max_len = (Array.length t1) in 
    let rec loop i = 
      if (i >= max_len) then 
	true
      else 
	if (t1.(i) < t2.(i)) then
	  loop (succ i)
	else
	  false
    in
    assert ((Array.length t1) = (Array.length t2));
    loop 0 

(* greater-than *)
  let gt t1 t2 = 
    let max_len = (Array.length t1) in 
    let rec loop i = 
      if (i >= max_len) then 
	true
      else 
	if (t1.(i) > t2.(i)) then
	  loop (succ i)
	else
	  false
    in
    assert ((Array.length t1) = (Array.length t2));
    loop 0 

(* greater-equal *)
  let ge t1 t2 = 
    let max_len = (Array.length t1) in 
    let rec loop i = 
      if (i >= max_len) then 
	true
      else 
	if (t1.(i) >= t2.(i)) then
	  loop (succ i)
	else
	  false
    in
    assert ((Array.length t1) = (Array.length t2));
    loop 0 


(* equal *)
  let equal t1 t2 = (not (lt t1 t2)) && (not (gt t1 t2))

(* strict equality *)
  let eq t1 t2 = (t1 = t2)

(* Increment our own counter *)
  let incr t org = t.(org) <- succ t.(org)

  let copy = Array.copy

  let sprint t = 
    Printf.sprintf "[%s]"
      (String.concat "," 
	 (Array.to_list (Array.map (fun x -> string_of_int x) t)))
end
   


(* Extend causalts with an order wrt. ensemble view *)

module VSCausal = struct 
  open Ensemble 
  open View 
    
  type s = Trans.ltime * Causalts.s
	
  let create_state (ls,vs) =
    (vs.ltime, Causalts.create_state ls.nmembers)

  let ge (ltime1,ts1) (ltime2,ts2) =
    (ltime1 > ltime2) || ((ltime1 = ltime2) && (Causalts.ge ts1 ts2)) 

  let incr (ltime,ts) org =  
    Causalts.incr ts org
      
  let copy (ltime,ts) = (ltime, Causalts.copy ts)

  let sprint (ltime,ts) = 
    Printf.sprintf "[%d%s]" ltime (Causalts.sprint ts)
end
  

