(* View independant identification of members *)
(* Dead members are never collected *)

(* Warning. (When used with the Log module), the mapping is not guaranteed 
   to be identic amongst member -- and is likely to differ  *)

module Name: sig 
  open Ensemble

  (* Returns a function mapping view dependant member rank to a view
     independant ranks.  The mapping is updated with the new view.

     TODO: Members quitting and then coming back are not handled
     correctly. A way should be to keep track of the uptime of the
     first singleton view of a member. But currently this information
     is loosed on view change. An another solution is to consider that
     member do not leave.  If they do leave and come back, they are
     considered as new member. This is true only if there is no group
     partition, which infortunatly may not hold.

     *)

  type map
  type fastmap = (Trans.rank -> Trans.rank)

   (* Returns a fresh new map *)
  val init_map : unit -> map 
      
   (* A map have to be updated each time a new member is joining *)
  val update_map  : map -> View.t -> map	

   (* Returns a fast mapping function:  view Dep rank -> View Indep index *)
  val get_fastmap : map -> View.t -> fastmap

end = struct
  open Ensemble

  type assoc = Endpt.id * int
  type map = assoc list
  type fastmap = (Trans.rank -> Trans.rank)

  let init_map () = []

  let update_map map view =
    let newmap = ref map in 
    Arrayf.iter
      (fun endpt -> 
	if not (List.mem_assoc endpt map) then
	  let index = (List.length map) in 
	  newmap := (endpt, index) :: !newmap)
      view;
    !newmap
      
  let get_fastmap map view =
    let getId i =
      let endpt = (Arrayf.get view i) in
      List.assoc endpt map
    in 
    let translateId = Array.init (Arrayf.length view) getId in
    (fun x -> translateId.(x))

end
