(* $Id: unixChrono.ml,v 1.3 1999/05/28 17:29:10 bruncott Exp $ *)
(* Unix chronos *)
(* Georges Brun-Cottan, Brandeis university, Jan 1999 *)

open Chrono
open Printf

module UnixElapsedTimeSet = struct
  type time = float
	
  let null  _ = 0.0
  let getTime = Unix.gettimeofday
  let sub t1 t2 = t1-.t2
  let add t1 t2 = t1+.t2

  let to_string t = sprintf "%f" t
end

module UnixPTimeSet = struct
  open Unix
  type time = Unix.process_times
	
  let null _ = { tms_utime  = 0.0;
		 tms_stime  = 0.0;
		 tms_cutime = 0.0;
		 tms_cstime = 0.0 } 
  let getTime = Unix.times
  let sub t1 t2 = 
      { tms_utime = t1.tms_utime -. t2.tms_utime;
	tms_stime = t1.tms_stime -. t2.tms_stime;
	tms_cutime = t1.tms_cutime -. t2.tms_cutime;
	tms_cstime = t1.tms_cstime -. t2.tms_cstime; }

  let add t1 t2 = 
      { tms_utime = t1.tms_utime +. t2.tms_utime;
	tms_stime = t1.tms_stime +. t2.tms_stime;
	tms_cutime = t1.tms_cutime +. t2.tms_cutime;
	tms_cstime = t1.tms_cstime +. t2.tms_cstime; }

  let to_string t = 
    sprintf "{user= %f, system= %f}" 
      t.tms_utime t.tms_stime
end

module UnixTimeSet = struct
  type time = Time of UnixElapsedTimeSet.time * UnixPTimeSet.time
	
  let null () = Time (UnixElapsedTimeSet.null(), UnixPTimeSet.null())
  let getTime () = Time (UnixElapsedTimeSet.getTime(), UnixPTimeSet.getTime())

  let sub t1 t2 =
    let Time(te1,tp1) = t1 in 
    let Time(te2,tp2) = t2 in 
    Time (UnixElapsedTimeSet.sub te1 te2, UnixPTimeSet.sub tp1 tp2)

  let add t1 t2 =
    let Time(te1,tp1) = t1 in 
    let Time(te2,tp2) = t2 in 
    Time (UnixElapsedTimeSet.add te1 te2, UnixPTimeSet.add tp1 tp2)
      
  let to_string t = 
    let Time(te,tp) = t in 
    sprintf "(elapsed= %s, %s)"
      (UnixElapsedTimeSet.to_string te)
      (UnixPTimeSet.to_string tp)
end


module UnixElapsedChrono =  Chrono(UnixElapsedTimeSet)
module UnixProcessChrono =  Chrono(UnixPTimeSet)

(* Unix Chronos are implemented as pairs of Chrono *)
module UnixChrono =  Chrono(UnixTimeSet)

    
